// test-qtl.cpp : ̨Ӧóڵ㡣
//

#ifndef WIN32
#define OutputDebugString printf
#include <iostream>
#include <algorithm>
using namespace std; 
#else 
#include "stdafx.h"
#endif 

#include <time.h>
#include <sstream>
#include <string>
#include "qtl/qtl_sqlite.hpp"

#define SQLITE_TIMEOUT 180000  // sqlite timeout
#define POPBOX_MSG_RETRY_MAX 5 // resend retry max

// for srvmsgs
class popbox_msg_t
{
public:
    void dump(char const* prompt) const;

    int msgtype = 0;   // 108 or 402
    int status = 0;    // send to server result, (1:ok; 0:fail)
    int count = 0;     // retry times, if exceed POPBOX_MSG_RETRY_MAX, stop retry
    time_t stamp = 0;  // receive time
    std::string msgid;
    std::string msgbody;
    std::string cid; 
    std::string uid; 
};


void popbox_msg_t::dump(char const* prompt) const
{
    tm* t = localtime(&stamp);
    printf("%s : %s,%s,%s,%d,%d,%d, %04d-%02d-%02d %02d:%02d:%02d, %s\n",
        prompt,
        cid.c_str(), 
        uid.c_str(), 
        msgid.c_str(),
        msgtype,
        status,
        count,
        t->tm_year + 1900,
        t->tm_mon + 1,
        t->tm_mday + 1,
        t->tm_hour,
        t->tm_min,
        t->tm_sec,
        msgbody.c_str());
}

namespace qtl
{
    template<>
    inline void bind_record<qtl::sqlite::statement, popbox_msg_t>(qtl::sqlite::statement& command, popbox_msg_t&& v)
    {
        int n = 0; 
        qtl::bind_field(command, n++, v.msgid);
        qtl::bind_field(command, n++, v.msgtype);
        qtl::bind_field(command, n++, v.cid);
        qtl::bind_field(command, n++, v.uid);
        qtl::bind_field(command, n++, v.status);
        qtl::bind_field(command, n++, v.count);
        qtl::bind_field(command, n++, v.msgbody);
        qtl::bind_field(command, n++, v.stamp);
    }
}
template <class OutputIterator>
int db_read_popbox_msg(OutputIterator it)
{
    int ret = 0;
    qtl::sqlite::database db(SQLITE_TIMEOUT);

    try
    {
        // copy of gcm.db, and create following table:
        // create table popmsg (msgid text not null, msgtype integer not null, cid text not null, uid text not null, status integer not null, count integer not null, msgbody text not null, stamp timestamp not null, primary key (msgid, msgtype, cid, uid));
        db.open("../data/gcm.db", NULL);
        printf("open db for resend popbox msg OK\n");

#if 0
        // add order by statement to ensure popbox ack(108) is added into queue before its feedback(402).
        db.query("select msgid, msgtype, cid, uid, status, count, msgbody, stamp from popmsg order by msgtype", 
            [&ret, &it](std::string const& msgid, int msgtype, std::string const& cid, std::string const& uid, int status, int count, std::string const& msgbody, time_t stamp) {
            popbox_msg_t pm;
            pm.msgid = msgid;
            pm.msgtype = msgtype;
            pm.cid = cid; 
            pm.uid = uid; 
            pm.status = status;
            pm.count = count;
            pm.msgbody = msgbody;
            pm.stamp = stamp;

            char buf[32] = { 0 };
            sprintf(buf, "popup msg %d", ret + 1);
            pm.dump(buf);

            *it = pm;
            ++ret;
            return true; 
        });
#else
        db.query("select msgid, msgtype, appname, uid, status, count, msgbody, stamp from popbox_msg order by msgtype", 
            [&ret, &it](popbox_msg_t const& pm) {
            char buf[512] = { 0 };
            sprintf(buf, "popup msg %d", ret + 1);
            pm.dump(buf);

            *it = pm;
            ++ret;
            return true;
        });
#endif

        db.close();
        printf("add %d popbox msg into delay queue\n", ret);
    }
    catch (qtl::sqlite::error &e)
    {
        printf("manipute db for resend popbox msg error %d: %s\n", e.code(), e.what());
        db.close();
        return -1;
    }

    return ret;
}


int main(int argc, char* argv[])
{
    std::vector <popbox_msg_t> vec; 
    db_read_popbox_msg(std::back_inserter(vec)); 
    printf("got %d iterms from db\n", vec.size()); 
    return 0; 
}
